package org.pvv.gnapplet;

/**
 * Class for handling Gnbus protocol responses.
 *
 * @author Håvard Wigtil
 */
public class GnbusResponse extends GnbusProtocolMessage {

    /** Error code for response. */
    private GnError error;

    /**
     * Private constructor. Use <code>create</code> methods to get new instances.
     *
     * @param type Response type
     * @param command Response command
     * @param data Response data
     */
    private GnbusResponse(byte type, short command, byte[] data) {
        super(type, command, data);
        error = GnError.NONE;
    }

    /**
     * Get status code from response.
     *
     * @return Status code
     */
    public GnError getStatus() {
        return error;
    }

    /**
     * Create a response to an info request.
     *
     * @param manufacturer Phone manufacturer
     * @param model Phone model
     * @param imei Phone IMEI or other ID
     * @param softwareVersion Phone software version
     * @param hardwareVersion Phone hardware revision
     * @return Response object
     */
    public static GnbusResponse createInfoResponse(String manufacturer, String model, String imei,
                                               String softwareVersion, String hardwareVersion) {
        ResponseData data = new ResponseData();
        data.append(manufacturer);
        data.append(model);
        data.append(imei);
        data.append(softwareVersion);
        data.append(hardwareVersion);

        return new GnbusResponse(Constants.GNAPPLET_MSG_INFO, Constants.GNAPPLET_MSG_INFO_ID_RESP, data.getBytes());
    }

    /**
     * Create a response to a power request.
     *
     * @param percent Battery charge, in percent
     * @param source Power source code
     * @return Response object
     */
    public static GnbusResponse createPowerResponse(byte percent, byte source) {
        ResponseData data = new ResponseData();
        data.append(percent);
        data.append(source);

        return new GnbusResponse(Constants.GNAPPLET_MSG_POWER, Constants.GNAPPLET_MSG_POWER_INFO_REQ, data.getBytes());
    }


    /**
     * Create a error response. Note that the command parameter is the request value, it will be increased by 1 to
     * get the corresponding response code.
     *
     * @param type Request type
     * @param command Request command
     * @param error Error code
     * @return Response object with error code
     */
    public static GnbusResponse createErrorResponse(byte type, short command, GnError error) {
        GnbusResponse response = new GnbusResponse(type, (short)(command + 1), new byte[0]);
        response.error = error;

        return response;
    }

    public static GnbusResponse createCommandResponse(byte type, short command) {
        return createErrorResponse(type, command, GnError.NONE);
    }

    /**
     * Get a response object as bytes.
     *
     * @return Response as byte array
     */
    public byte[] toBytes() {
        byte[] result = new byte[16 + data.length];
        result[0] = Constants.MAGIC_BYTE;
        writeShort(result, 2, (short)(result.length - 8)); // Header and checksum doesn't count towards message size
        result[4] = type;
        writeShort(result, 6, command);
        writeShort(result, 8, (short)error.ordinal());
        writeShort(result, 10, Constants.GNAPPLET_MAJOR_VERSION);
        writeShort(result, 12, Constants.GNAPPLET_MINOR_VERSION);

        System.arraycopy(data, 0, result, 14, data.length);

        byte[] checksum = generateChecksumBytes(result);
        System.arraycopy(checksum, 0, result, result.length - 2, 2);

        return result;
    }

    /**
     * Write a short as high and low byte to an array.
     *
     * @param array The array to write to
     * @param index Position for high byte, low byte will be index + 1
     * @param s The value to write
     */
    private static void writeShort(byte[] array, int index, short s) {
        array[index] = (byte)(s >>> 8);
        array[index + 1] = (byte)(s & 0xff);
    }
}
