package org.pvv.gnapplet;

import android.app.Service;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.telephony.SmsManager;
import android.telephony.TelephonyManager;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;

/**
 * Thread for handling a single request.
 *
 * @author Håvard Wigtil
 */
public class RequestThread extends Thread {

    /** Logger tag. */
    private static final String TAG = "Gnapplet.RequestThread";

    /** Socket for communication. */
    private final BluetoothSocket socket;

    /** Instance of service that owns this thread. */
    private final Service service;

    /**
     * Create a new instance.
     *
     * @param socket Socket for this request
     * @param service Parent service
     */
    public RequestThread(BluetoothSocket socket, Service service) {
        this.socket = socket;
        this.service = service;
    }

    /**
     * Read a request from the socket and return a response, then  close the socket.
     */
    @Override
    public void run() {
        // Try block for socket resource
        try {

            while (true) {
                Log.d(TAG, "Reading request");
                GnbusRequest request = getRequest(socket);

                if (request == null) {
                    // Fatal error in request or no more requests
                    Log.d(TAG, "No more requests, exiting event loop");
                    break;
                }
                Log.i(TAG, "Request: " + request);

                GnbusResponse response = createResponse(request);

                byte[] output = response.toBytes();
                Log.d(TAG, "Returning response: " + toHex(output));

                socket.getOutputStream().write(output);
            }

        } catch (Exception e) {
            Log.e(TAG, "Unhandled exception in run()", e);
        } finally {
            try {
                Log.d(TAG, "Closing socket");
                socket.close();
            } catch (IOException e) {
                Log.w(TAG, "Failed to close socket");
            }
        }

    }

    public static GnbusRequest getRequest(final BluetoothSocket socket) {
        try {
            byte[] input = readInput(socket.getInputStream());
            Log.d(TAG, "Incoming request: " + toHex(input));
            return GnbusRequest.parse(input);
        } catch (IOException e) {
            Log.e(TAG, "Failed to read input", e);
            return null;
        } catch (IllegalArgumentException e) {
            Log.e(TAG, "Failed to parse input", e);
            // TODO Maybe return an error response
            return null;
        }

    }

    private GnbusResponse createResponse(GnbusRequest request) {
        GnbusResponse response;
        try {
            switch (request.getType())
            {
                case Constants.GNAPPLET_MSG_INFO:
                    TelephonyManager mgr = (TelephonyManager)service.getSystemService(Context.TELEPHONY_SERVICE);
                    response = new InfoRequestHandler(request, mgr).createResponse();
                    break;

                case Constants.GNAPPLET_MSG_SMS:
                    SmsRequestHandler smshandler = new SmsRequestHandler(request, SmsManager.getDefault(),
                                                                         service);
                    response = smshandler.createResponse();
                    break;

                case Constants.GNAPPLET_MSG_POWER:
                    response = new PowerRequestHandler(request, service).createResponse();
                    break;

                default:
                    Log.w(TAG, String.format("Unsupported request type %d", request.getType()));
                    response = GnbusResponse.createErrorResponse(request.getType(),
                                                                 request.getCommand(),
                                                                 GnError.NOTIMPLEMENTED);
            }

        } catch (Exception e) {
            String msg = String.format("Unhandled exception (%s) when creating response: %s",
                                       e.getClass().getName(), e.getMessage());
            Log.e(TAG, msg, e);
            response = GnbusResponse.createErrorResponse(request.getType(), request.getCommand(),
                                                        GnError.FAILED);
        }

        return response;
    }

    private static byte[] readInput(InputStream is) throws IOException {
        byte[] buffer = new byte[1024];

        int read = is.read(buffer);
        Log.d(TAG, String.format("Read %d bytes from input", read));

        // For now we'll always get the complete message in the buffer
        if (read >= 1024) {
            throw new RuntimeException("Input is larger than 1024 bytes, time to do some more coding!");
        }

        return buffer;
    }

    /**
     * Convert a byte array to a list of hex values, separated by space.
     *
     * @param bytes The array to convert
     * @return List of hex bytes
     */
    private static String toHex(byte[] bytes) {
        if (bytes == null) {
            return "null";
        }
        if (bytes.length == 0) {
            return "";
        }

        StringBuilder sb = new StringBuilder(bytes.length * 5);
        for (byte b : bytes) {
            sb.append(String.format("%02x ", b));
        }
        sb.deleteCharAt(sb.length() - 1);

        return sb.toString();
    }
}
