package org.pvv.gnapplet;

import java.io.UnsupportedEncodingException;

/**
 * Class for writing data in Gnbus format.
 *
 * @author Håvard Wigtil
 */
public class ResponseData {

    /** Default new buffer and buffer increment size. */
    private static final int DEFAULT_INCREMENT = 1024;

    /** Data buffer. */
    byte[] data;

    /** Current content length. */
    int length;

    /**
     * Create a new instance with data buffer size set.
     *
     * @param initialSize Buffer size
     */
    public ResponseData(int initialSize) {
        this.length = 0;
        data = new byte[initialSize];
    }

    /**
     * Create a new instance with default buffer size.
     */
    public ResponseData() {
        this(DEFAULT_INCREMENT);
    }

    /**
     * Get data as a byte array.
     *
     * @return Data array, trimmed to length
     */
    public byte[] getBytes() {
        byte[] result = new byte[length];
        System.arraycopy(data, 0, result, 0, length);
        return result;
    }

    /**
     * Make sure the data buffer can hold the given number of new bytes.
     * This method will expand the buffer if needed.
     *
     * @param count The number of extra bytes the buffer should hold
     */
    private void ensureCapacity(int count) {

        if (this.length  + count> data.length) {
            byte[] tmp = new byte[data.length + Math.max(DEFAULT_INCREMENT, count)];
            System.arraycopy(data, 0, tmp, 0, data.length);
            data = tmp;
        }
    }

    /**
     * Add a byte to the end of the buffer.
     *
     * @param b The byte to add
     */
    public void append(byte b) {
        ensureCapacity(1);
        data[length] = b;
        length += 1;
    }

    /**
     * Add a short to the ned of the buffer.
     *
     * @param s The short to add
     */
    public void append(short s) {
        ensureCapacity(2);
        data[length] = (byte)(s >>> 8);
        data[length + 1] = (byte)(s & 0xff);
        length += 2;
    }

    /**
     * Add a String to the end of the buffer. The string will be UTF-16BE encoded, after two bytes for length.
     *
     * @param string The string to add
     */
    public void append(String string) {
        try {
            byte[] bytes = string.getBytes("UTF-16BE");
            ensureCapacity(bytes.length + 2);
            // Note that lengt is number of UTF-16 chars, not number of bytes
            append((short)string.length());
            copyBytes(bytes);
        } catch (UnsupportedEncodingException e) {
            throw new RuntimeException("Encoding UTF-16BE is not supported. This should be impossible.", e);
        }
    }

    /**
     * Add an array of bytes to the end of the buffer.
     *
     * @param bytes The array to add
     */
    public void append(byte[] bytes) {
        ensureCapacity(bytes.length + 2);
        append((short)bytes.length);
        copyBytes(bytes);
    }

    /**
     * Copy an array of bytes to the end of the buffer without adding any length information first.
     *
     * @param bytes The array to copy
     */
    private void copyBytes(byte[] bytes) {
        System.arraycopy(bytes, 0, data, length, bytes.length);
        length += bytes.length;
    }
}
