package org.pvv.gnapplet;

import android.app.Service;
import android.content.ContentValues;
import android.content.pm.ProviderInfo;
import android.net.Uri;
import android.telephony.SmsManager;
import android.telephony.SmsMessage;
import android.util.Log;
import org.ajwcc.pduUtils.gsm3040.Pdu;
import org.ajwcc.pduUtils.gsm3040.PduParser;
import org.ajwcc.pduUtils.gsm3040.PduUtils;

import java.util.Arrays;

/**
 * Handler for sms requests.
 *
 * @author Håvard Wigtil
 */
public class SmsRequestHandler extends AbstractRequestHandler {

    /** Logger tag. */
    private static final String TAG = "Gnapplet.SmsRequestHandler";

    /** SMS manager instance. */
    private SmsManager manager;

    /** Service instance. */
    private Service service;

    /**
     * Create a new instance.
     *
     * @param request Request object
     * @param manager SMS manager instance
     * @param service Parent service instance
     */
    public SmsRequestHandler(GnbusRequest request, SmsManager manager, Service service) {
        super(request, TAG);
        this.manager = manager;
        this.service = service;
    }

    /**
     * Create a response to an SMS request.
     *
     * @return The response
     */
    @Override
    public GnbusResponse createResponse() {
        switch (getRequest().getCommand()) {
            case Constants.GNAPPLET_MSG_SMS_SEND_REQ:
                return sendSms();

            default:
                return handleUnsupportedCommand();
        }
    }

    /**
     * Send an SMS.
     *
     * @return Send SMS response
     */
    private GnbusResponse sendSms() {
        RequestData requestData = new RequestData(getRequest().data);
        byte[] rawSms = requestData.readByteArray();

        Log.d(TAG, String.format("Raw data is %d bytes: %s", rawSms.length, Arrays.toString(rawSms)));

        /* Using SMSLib below because SmsMessage.createFromPdu returns null on Xperia Pro with Android 2.3.4.
          Not sure if this is vendor, version or other problems. createFromPdu works on HTC Hero with 2.1.1
          and HTC Desire with 2.2. Logging result to check other devices and versions. */
        SmsMessage sms = SmsMessage.createFromPdu(rawSms);
        Log.d(TAG, "PDU parsed with SmsMessage: " + sms);

        // Get data from PDU
        PduParser parser = new PduParser();
        Pdu pdu = parser.parsePdu(PduUtils.bytesToPdu(rawSms));
        String message = pdu.getDecodedText();
        String recipient = pdu.getAddress();

        Log.i(TAG, String.format("Sending SMS '%s' to %s", message, recipient));
        manager.sendTextMessage(recipient, null, message, null, null);

        // TODO Check for send status and only write response on success
        saveSentMessage(recipient, message);

        return GnbusResponse.createCommandResponse(Constants.GNAPPLET_MSG_SMS, Constants.GNAPPLET_MSG_SMS_SEND_REQ);
    }

    /**
     * Write a message to the "sent" folder.
     *
     * @param recipient Message recipient
     * @param message Message body
     */
    private void saveSentMessage(String recipient, String message) {
        // Might break on some phones and future OS versions, but there is no standard way to do this
        ProviderInfo pi = service.getPackageManager().resolveContentProvider("sms", 0);
        if (pi != null) {
            Log.i(TAG, "Writing message to sent folder");
            ContentValues values = new ContentValues();
            values.put("address", recipient);
            values.put("body", message);
            service.getContentResolver().insert(Uri.parse("content://sms/sent"), values);
        } else {
            Log.w(TAG, "No known content provider for SMS, can't store message");
        }

    }

}
